@@ -7,7 +7,7 @@ import bitformat
 from ._common import Expression, Endianness, byteorder, DtypeKind, override, final, parser_str, ExpressionError
 from lark import Transformer, UnexpectedInput
 import lark
-from bitformat.bit_rust import Bits, bits_from_any
+from bitformat.rust import Bits, bits_from_any
 
 # Things that can be converted to Bits when a Bits type is needed
 BitsType = Union["Bits", str, Iterable[Any], bytearray, bytes, memoryview]
@@ -151,13 +151,23 @@ class Dtype(abc.ABC):
         ...
 
     @abc.abstractmethod
+    def _unpack(self, b: BitsType, /) -> tuple[int, Any]:
+        """Unpacks a Bits, returning number of bits unpacked as first element of tuple"""
+        ...
+
+    @abc.abstractmethod
+    def _unpack_no_checks(self, b: Bits | MutableBits) -> Any | tuple[Any]:
+        """Unpacks the whole of a Bits, without checks on length or type."""
+        ...
+
     def unpack(self, b: BitsType, /):
         """Unpack a Bits to find its value.
 
         The b parameter should be a Bits of the appropriate length, or an object that can be converted to a Bits.
 
         """
-        ...
+        return self._unpack(b)[1]  # We only return the values, not the number of bits unpacked.
+
 
     @abc.abstractmethod
     def _get_bit_length(self) -> int | None:
@@ -268,6 +278,8 @@ class Dtype(abc.ABC):
     def __hash__(self) -> int:
         ...
 
+    def _is_padding(self) -> bool:
+        return False
 
 
 class DtypeSingle(Dtype):
@@ -278,7 +290,6 @@ class DtypeSingle(Dtype):
 
     """
 
-    _kind: DtypeKind
     _size: Expression
     _bit_length: int | None
     _definition: DtypeDefinition
@@ -286,6 +297,10 @@ class DtypeSingle(Dtype):
     _create_fn: Callable[[Any], Bits]
     _get_fn: Callable[[bitformat.Bits], Any]
 
+    @override
+    def _is_padding(self) -> bool:
+        return self.kind == DtypeKind.PAD
+
     @override
     def info(self) -> str:
         if self.bit_length is not None:
@@ -329,7 +344,7 @@ class DtypeSingle(Dtype):
             else:
                 x._bit_length = x._size.const_value * definition.bits_per_character
         little_endian = (endianness is Endianness.LITTLE or
-                     (endianness is Endianness.NATIVE and bitformat.byteorder == "little"))
+                     (endianness is Endianness.NATIVE and bitformat.byteorder is Endianness.LITTLE))
         x._endianness = endianness
 
         if little_endian:
@@ -351,7 +366,7 @@ class DtypeSingle(Dtype):
             bs = x._set_fn(v, length=x._bit_length)
             mutable = bs.to_mutable_bits()  # TODO: Do we really need to clone here?
             mutable.byte_swap()
-            return mutable._as_immutable()
+            return mutable.as_bits()
         x._create_fn = create_bits_le if little_endian else create_bits
         return x
 
@@ -385,17 +400,22 @@ class DtypeSingle(Dtype):
 
     @override
     @final
-    def unpack(self, b: BitsType, /) -> Any | tuple[Any]:
+    def _unpack(self, b: BitsType, /) -> tuple[int, Any | tuple[Any]]:
         b = bits_from_any(b)
         if self._size.is_none():
             # Try to unpack everything
-            return self._get_fn(b, 0, len(b))
+            return len(b), self._get_fn(b, 0, len(b))
         if self._bit_length is None:
             raise ExpressionError(f"Cannot unpack a dtype with an unknown size. Got '{self}'")
         if self._bit_length > len(b):
             raise ValueError(f"{self!r} is {self._bit_length} bits long, but only got {len(b)} bits to unpack.")
         else:
-            return self._get_fn(b, 0, self._bit_length)
+            return self._bit_length, self._get_fn(b, 0, self._bit_length)
+
+    @override
+    @final
+    def _unpack_no_checks(self, b: Bits | MutableBits) -> Any | tuple[Any]:
+        return self._get_fn(b, 0, len(b))
 
     @override
     @final
@@ -518,23 +538,31 @@ class DtypeArray(Dtype):
 
     @override
     @final
-    def unpack(self, b: BitsType, /) -> Any | tuple[Any]:
+    def _unpack(self, b: BitsType, /) -> tuple[int, Any | tuple[Any]]:
         b = bits_from_any(b)
         if self.items is not None and self.bit_length is not None and self.bit_length > len(b):
-            raise ValueError(f"{self!r} is {self.bit_length} bits long, but only got {len(b)} bits to unpack.")
+            raise ValueError(f"{self!r} is {self.bit_length} bits long, but only {len(b)} bits are available.")
         items = self._items.evaluate()
         if self._dtype_single.bit_length is None:
-            raise ValueError(f"Cannot unpack when the DtypeSingle has an unknown size. Got '{self}'")
+            raise ValueError(f"Cannot read or unpack a DtypeSingle with an unknown size. Got '{self}'")
         if self._items.is_none():
             # For array dtypes with no items (e.g. '[u8;]') unpack as much as possible.
             if self._dtype_single.bit_length is None:
-                raise ValueError(f"Cannot unpack when DtypeArray items is unspecified and the DtypeSingle has an unknown size. Got '{self}'")
+                raise ValueError(f"Cannot read or unpack a DtypeArray if items is unspecified and the DtypeSingle has an unknown size. Got '{self}'")
             items = len(b) // self._dtype_single.bit_length
-        return tuple(
+        return (items * self._dtype_single.bit_length), tuple(
             self._dtype_single.unpack(b[i * self._dtype_single.bit_length : (i + 1) * self._dtype_single.bit_length])
             for i in range(items)
         )
 
+    @override
+    @final
+    def _unpack_no_checks(self, b: Bits | MutableBits) -> tuple[Any]:
+        if isinstance(b, bitformat.MutableBits):
+            b = b.to_bits()
+        return tuple(self._dtype_single._unpack_no_checks(c) for c in b.chunks(self._dtype_single.bit_length, count=self.items))
+
+
     @override
     @final
     def __str__(self) -> str:
@@ -662,11 +690,11 @@ class DtypeTuple(Dtype):
         """
         if len(values) != self.items:
             raise ValueError(f"Expected {self.items} values, but got {len(values)}.")
-        return bitformat.Bits._from_joined([dtype.pack(value) for dtype, value in zip(self._dtypes, values)])
+        return bitformat.Bits.from_joined([dtype.pack(value) for dtype, value in zip(self._dtypes, values)])
 
     @override
     @final
-    def unpack(self, b: bitformat.Bits | str | Iterable[Any] | bytearray | bytes | memoryview, /) -> tuple[tuple[Any] | Any]:
+    def _unpack(self, b: bitformat.Bits | str | Iterable[Any] | bytearray | bytes | memoryview, /) -> tuple[int, tuple[tuple[Any] | Any]]:
         """Unpack a Bits to find its value.
 
         The b parameter should be a Bits of the appropriate length, or an object that can be converted to a Bits.
@@ -683,6 +711,23 @@ class DtypeTuple(Dtype):
                 raise ValueError(f"{self!r} is {self.bit_length} bits long, but only got {len(b)} bits to unpack.")
         vals = []
         pos = 0
+        for i, dtype in enumerate(self._dtypes):
+            if i == self._dynamic_index:
+                dynamic_length = len(b) - self._bit_length
+                vals.append(dtype.unpack(b[pos : pos + dynamic_length]))
+                pos += dynamic_length
+            else:
+                x = dtype.unpack(b[pos : pos + dtype.bit_length])
+                if x is not None:  # Padding could unpack as None
+                    vals.append(x)
+                pos += dtype.bit_length
+        return pos, tuple(vals)
+
+    @override
+    @final
+    def _unpack_no_checks(self, b: Bits | MutableBits) -> tuple[Any]:
+        pos = 0
+        vals = []
         for i, dtype in enumerate(self._dtypes):
             if i == self._dynamic_index:
                 dynamic_length = len(b) - self._bit_length
@@ -695,6 +740,8 @@ class DtypeTuple(Dtype):
                 pos += dtype.bit_length
         return tuple(vals)
 
+
+
     @override
     @final
     def _get_bit_length(self) -> int | None:
@@ -903,12 +950,18 @@ class Register:
     def add_dtype(cls, definition: DtypeDefinition):
         kind = definition.kind
         cls.kind_to_def[kind] = definition
-        def fget(obj):
-            return definition.get_fn(obj, 0, len(obj))
-        setattr(bitformat.Bits, kind.value, property(fget=fget,
+
+        def fget_bitwise(b):
+            return definition.get_fn(b, 0, len(b))
+
+        setattr(bitformat.Bits, kind.value, property(fget=fget_bitwise,
                                                      doc=f"The Bits as {definition.description}. Read only."))
-        setattr(bitformat.MutableBits, kind.value, property(fget=fget,
-                                                     doc=f"The MutableBits as {definition.description}. Read only."))
+
+        def fset_bitwise(b, val):
+            b[:] = definition.set_fn(val, length=len(b))
+
+        setattr(bitformat.MutableBits, kind.value, property(fget=fget_bitwise, fset=fset_bitwise,
+                                                            doc=f"The MutableBits as {definition.description}. Read and write."))
 
         if definition.endianness_variants:
 
@@ -917,34 +970,46 @@ class Register:
                     raise ValueError(f"Cannot use endianness modifer for non whole-byte data. Got length of {len(b)} bits.")
                 return definition.get_fn(b, 0, len(b))
 
-            def fget_le_bits(b):
+            def fget_le(b):
                 if len(b) % 8 != 0:
                     raise ValueError(f"Cannot use endianness modifer for non whole-byte data. Got length of {len(b)} bits.")
                 mutable_b = b.to_mutable_bits()
                 mutable_b.byte_swap()
                 return definition.get_fn(mutable_b.to_bits(), 0, len(b))
 
-            def fget_le_mutable_bits(b):
-                if len(b) % 8 != 0:
-                    raise ValueError(f"Cannot use endianness modifer for non whole-byte data. Got length of {len(b)} bits.")
-                c = b.__copy__()  # TODO: Not sure we really need a copy here.
-                c.byte_swap()
-                return definition.get_fn(c._as_immutable(), 0, len(b))
+            fget_ne = fget_le if byteorder is Endianness.LITTLE else fget_be
 
-            fget_ne_bits = fget_le_bits if byteorder == "little" else fget_be
-            fget_ne_mutable_bits = fget_le_mutable_bits if byteorder == "little" else fget_be
-
-            for modifier, fget, desc in [("_le", fget_le_bits, "little-endian"),
+            for modifier, fget, desc in [("_le", fget_le, "little-endian"),
                                          ("_be", fget_be, "big-endian"),
-                                         ("_ne", fget_ne_bits, f"native-endian (i.e. {byteorder}-endian)")]:
+                                         ("_ne", fget_ne, f"native-endian (i.e. {byteorder}-endian)")]:
                 doc = f"The Bits as {definition.description} in {desc} byte order. Read only."
                 setattr(bitformat.Bits, kind.value + modifier, property(fget=fget, doc=doc))
 
-            for modifier, fget, desc in [("_le", fget_le_mutable_bits, "little-endian"),
-                                         ("_be", fget_be, "big-endian"),
-                                         ("_ne", fget_ne_mutable_bits, f"native-endian (i.e. {byteorder}-endian)")]:
-                doc = f"The MutableBits as {definition.description} in {desc} byte order. Read only."
-                setattr(bitformat.MutableBits, kind.value + modifier, property(fget=fget, doc=doc))
+            def fget_le_mut(b):
+                if len(b) % 8 != 0:
+                    raise ValueError(f"Cannot use endianness modifer for non whole-byte data. Got length of {len(b)} bits.")
+                c = b.__copy__()
+                c.byte_swap()
+                return definition.get_fn(c.as_bits(), 0, len(b))
+
+            def fset_le(b, v):
+                if len(b) % 8 != 0:
+                    raise ValueError(f"Cannot use endianness modifer when setting non whole-byte data. Got length of {len(b)} bits.")
+                b[:] = definition.set_fn(v, len(b)).to_mutable_bits().byte_swap()
+
+            def fset_be(b, v):
+                if len(b) % 8 != 0:
+                    raise ValueError(f"Cannot use endianness modifer when setting non whole-byte data. Got length of {len(b)} bits.")
+                b[:] = definition.set_fn(v, len(b))
+
+            fget_ne_mut = fget_le_mut if byteorder is Endianness.LITTLE else fget_be
+            fset_ne = fset_le if byteorder is Endianness.LITTLE else fset_be
+
+            for modifier, fget, fset, desc in [("_le", fget_le_mut, fset_le, "little-endian"),
+                                               ("_be", fget_be, fset_be, "big-endian"),
+                                               ("_ne", fget_ne_mut, fset_ne, f"native-endian (i.e. {byteorder}-endian)")]:
+                doc = f"The MutableBits as {definition.description} in {desc} byte order. Read and write."
+                setattr(bitformat.MutableBits, kind.value + modifier, property(fget=fget, fset=fset, doc=doc))
 
     @classmethod
     @functools.lru_cache(CACHE_SIZE)
