@@ -387,7 +387,7 @@ class TestEndiannesssEdgeCases:
         native_endian = Bits.from_dtype("u16_ne", value)
         
         # They should be different (unless native happens to match one)
-        if sys.byteorder == 'big':
+        if sys.byteorder is Endianness.BIG:
             assert big_endian == native_endian
         else:
             assert little_endian == native_endian
@@ -500,7 +500,7 @@ class TestHypothesisPropertyBasedEdgeCases:
         
         # Should be able to unpack back to original values
         recovered = result.unpack([f"u8"] * len(byte_values))
-        assert recovered == byte_values
+        assert recovered == tuple(byte_values)
         
     @given(st.binary(min_size=1, max_size=50))
     @settings(max_examples=30)
