@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
-from bitformat import Format, Dtype, DtypeSingle, Bits, Field, Array, FieldType, Repeat, DtypeKind, Expression, While
+from bitformat import Format, Dtype, DtypeSingle, Bits, Field, Array, FieldType, Repeat, DtypeKind, Expression, While, \
+    MutableBits
 from hypothesis import given
 import pytest
 import hypothesis.strategies as st
@@ -311,11 +312,11 @@ def test_repeat_with_const_expression():
     assert f.to_bits() == "i9=3, 0x070707"
 
 def test_repeat_with_bits():
-    f = Repeat.from_params(3, "bits=0xab")
+    f = Repeat.from_params(3, "const bits=0xab")
     f.pack([])
     b = f.to_bits()
     assert b == "0xababab"
-    f2 = Repeat.from_params(2, Field.from_bits(b))
+    f2 = Repeat.from_params(2, Field.from_bits(b, const=True))
     f2.pack([])
     b2 = f2.to_bits()
     assert b2 == "0xabababababab"
@@ -382,7 +383,6 @@ def test_from_string():
     assert str(f) == str(Format(str(f)))
 
 
-@pytest.mark.skip  # TODO
 def test_recursive_from_string():
     s = "header: (u8, u4, bool,body:(u8=23, [u4; 3], bool))"
     f = FieldType.from_string(s)
@@ -600,7 +600,7 @@ def test_wrong_arguments():
     f.clear()
     with pytest.raises(TypeError):
         f.pack(1)
-    with pytest.raises(TypeError):
+    with pytest.raises(ValueError):
         f.pack([1])
 
 def test_slicing_fields():
@@ -767,20 +767,18 @@ def test_format_with_repeat():
     assert fmt.value[0] == 2  # count
     assert fmt.value[1] == [1, 2]  # values
 
-@pytest.mark.skip  #TODO
 def test_format_with_conditional_fields():
     """Test a format with conditional fields based on a flag"""
     fmt = Format("(has_name: bool, if {has_name}: name: bytes4, id: u16)")
 
     # Test with name (has_name=True)
     fmt.pack([True, b"ABCD", 1234])
-    assert fmt.to_bits() == Bits.from_string("0b1, b'ABCD', u16=1234")
+    assert fmt.to_bits() == Bits('0b1') + b'ABCD' + 'u16 = 1234'
 
     # Test without name (has_name=False)
     fmt.pack([False, 5678])  # No name needed
     assert fmt.to_bits() == Bits.from_string('0b0, u16=5678')
 
-@pytest.mark.skip  # TODO
 def test_format_with_complex_expressions():
     """Test a format with more complex expressions"""
     fmt = Format("(width: u8, height: u8, repeat{width*height}: pixel: u8)")
@@ -794,7 +792,6 @@ def test_format_with_complex_expressions():
     fmt.parse(data)
     assert fmt.value == [1, 2, [1, 2]]
 
-@pytest.mark.skip  # TODO
 def test_format_with_padding():
     """Test a format with padding fields"""
     fmt = Format("(a: u4, pad4, b: u8)")
@@ -830,7 +827,6 @@ def test_format_with_dynamic_dtype():
     fmt.pack([3, True])
     assert fmt.to_bits() == Bits.from_string("0b111")
 
-@pytest.mark.skip  # TODO
 def test_format_with_expressions_in_if_condition():
     """Test a format with expressions in if condition that reference other fields"""
     fmt = Format("(flags: u8, len: u8, if {flags & 0x01 != 0}: opt1: u8, if {flags & 0x02 != 0}: opt2: u16)")
@@ -892,10 +888,15 @@ def test_more_while():
     v = f.unpack(b, x=2)
     assert v[0] == [1, 255, 2]
 
-def test_pack_with_kwargs():
-    f = Format("(x: u8, y: u8, z: u8)")
-    f.pack([1, 2, 3])
-    assert f.value == [1, 2, 3]
-    f.clear()
-    f.pack([], x=4, y=5, z=6)
-    assert f.value == [4, 5, 6]
+def test_appending_fields():
+    f = Format()
+    f += "x: u8"
+    assert f.parse('0x10') == 8
+    x_value = f['x'].value
+    assert x_value == 0x10
+
+def test_string_value():
+    f = Format("(x: const hex4 = dead, if {x == 'dead'}: u8)")
+    b = MutableBits('0xdeadff')
+    f.parse(b)
+    assert f.value == ['dead', 255]
\ No newline at end of file
