@@ -189,8 +189,10 @@ def test_delitem_slice():
 def test_delitem_with_step():
     # Test slices with step
     a = MutableBits('0b101010')
+    del a[::2]  # Delete every other bit
+    assert a == '0b000'
     with pytest.raises(ValueError):
-        del a[::2]  # Delete every other bit
+        del a[::0]
 
 
 def test_delitem_edge_cases():
@@ -998,4 +1000,94 @@ def test_shifts():
 def test_str():
     a = MutableBits.from_ones(8)
     assert a.__str__() == '0xff'
-    assert a.__repr__() == "MutableBits('0xff')"
\ No newline at end of file
+    assert a.__repr__() == "MutableBits('0xff')"
+
+def test_logical_op_misc():
+    a = MutableBits('0xffff')
+    b = MutableBits('0x000')
+    try:
+        _ = a & b
+    except ValueError as e:
+        assert "12" in str(e)
+        assert "16" in str(e)
+
+def test_auto_conversions():
+    a = MutableBits()
+    with pytest.raises(TypeError):
+        _ = a + None
+    with pytest.raises(TypeError):
+        _ = a + True
+    with pytest.raises(TypeError):
+        _ = a + False
+    with pytest.raises(TypeError):
+        _ = a + 1
+    b = a + '0x1'
+    assert isinstance(b, MutableBits) and b == '0x1'
+    b = a + b'123'
+    assert isinstance(b, MutableBits) and b == b'123'
+    b = a + [1, 0]
+    assert isinstance(b, MutableBits) and b == '0b10'
+    b = a + (1, 0, 'steve')
+    assert isinstance(b, MutableBits) and b == '0b101'
+
+def test_clear():
+    a = MutableBits()
+    a.clear()
+    assert a == MutableBits()
+    assert not a
+    a += '0b1'
+    assert a
+    a.clear()
+    assert not a
+    assert a == MutableBits()
+
+def test_reserve():
+    a = MutableBits()
+    assert a.capacity() == 0
+    a.reserve(10)
+    assert a.capacity() >= 10
+    a += MutableBits.from_random(1000000)
+    b4 = a.capacity()
+    assert b4 >= 1000000
+    a.clear()
+    assert a.capacity() == b4
+
+def test_assigning_by_properties():
+    a = MutableBits.from_zeros(64)
+    a.i = -1
+    assert a == MutableBits.from_ones(64)
+    a.f_le = -0.25
+    assert a.f_le == -0.25
+    assert len(a) == 64
+    a.u_be = 123
+    assert a.u_be == 123
+    a.i_ne = 55
+    assert a.i_ne == 55
+
+def test_insert_slice():
+    a = MutableBits('0xff')
+    a[0:0] = '0xab'
+    assert a == '0xabff'
+    a[0:0] = a
+    assert a == '0xabffabff'
+
+def test_del_ranges():
+    a = MutableBits.from_zeros(10)
+    del a[5:3]
+    assert len(a) == 10
+
+def test_set_item_with_step():
+    a = MutableBits('0b000000')
+    a[::2] = '0b110'
+    assert a == '0b101000'
+
+def test_iter():
+    a = MutableBits('0b110')
+    with pytest.raises(TypeError):
+        _ = [bool(q) for q in a]
+
+def test_to_bytes_bug():
+    a = MutableBits.from_string('0xff')
+    b = a[2:]
+    c = b.to_bytes()
+    assert c == b'\xfc'
\ No newline at end of file
